
var app = angular.module('VME_Module', ['ngFileUpload', 'ui.bootstrap', 'ngIdle']);

app.constant('SYSTEM_PATH', '/SystemDS');
app.constant('CONFIG_PATH', '/ConfigDS');
app.constant('CAMPAIGN_PATH', '/CampaignDs');
app.constant('FLEET_PATH', '/FleetDs');
app.constant('LEASING_PATH', '/LeasingDS');
app.constant('INSURANCE_PATH', '/InsuranceDS');
app.constant('DEALER_PATH', '/DealerPortalDs');
app.constant('INCENTIVE_PATH', '/IncentiveDS');
app.constant('CQ_PATH', '/CQDS');
app.constant('ACCOUNT_PATH', '/AccountDS');
app.constant('INTERFACE_PATH', '/InterfaceDs');
app.constant('MS_PATH', '/MSControlDS');
app.constant('DEMOCAR_PATH', '/DemoCarDS');
app.constant('REPORT_PATH', '/WSYS/Report.aspx?');

app.config(['$httpProvider', function ($httpProvider) {
        // Initialize get if not there
        if (!$httpProvider.defaults.headers.get) {
            $httpProvider.defaults.headers.get = {};
        }

        // Enables Request.IsAjaxRequest() in ASP.NET MVC
        $httpProvider.defaults.headers.common["X-Requested-With"] = 'XMLHttpRequest';

        // Disable IE ajax request caching
        $httpProvider.defaults.headers.get['If-Modified-Since'] = '0';
}]);

app.config(['IdleProvider', 'KeepaliveProvider', function (IdleProvider, KeepaliveProvider) {
    //Idle if no action on page.
    IdleProvider.idle(20*60); // Idle for 20 min
    IdleProvider.timeout(5*60); // Timeout after idle 5 min

    //Comment KeepaliveProvider if no need to keep session alive
    KeepaliveProvider.interval(20*60); // heartbeat every 20 min
    KeepaliveProvider.http('/System/Heartbeat'); // URL that makes sure session is alive
}]);

app.config(['uibDatepickerConfig', function (uibDatepickerConfig) {
    uibDatepickerConfig.showWeeks = false;
}]);

app.config(['uibDatepickerPopupConfig', function (uibDatepickerPopupConfig) {
    // uibDatepickerPopupConfig.showButtonBar = false;
    uibDatepickerPopupConfig.closeOnDateSelection = false;
}]);

app.run(['$rootScope', '$window', 'BaseService', 'Idle', function ($rootScope, $window, BaseService, Idle) {
    //Check the idle state
    if($window.location.pathname != "/") {
        Idle.watch();
        $rootScope.$on('IdleTimeout', function () {
            BaseService.Message.alert("Sesstion Timeout").then(function () {
                window.location.href = "/";
            })
        })
    }
}]);

app.directive('loading', ['$http' ,function ($http)
    {
        return {
            restrict: 'A',
            link: function (scope, element, attrs)
            {
                scope.isLoading = function () {
                    return $http.pendingRequests.length > 0;
                };

                scope.$watch(scope.isLoading, function (v)
                {
                    if(v){
                        document.activeElement.blur();
                        element.show();
                    }else{
                        element.hide();
                    }
                });
            }
        };

}]);

app.directive('paginationTable', ['$compile', function ($compile) {
    return {
        restrict: 'AE',
        link: function (scope, element, attrs, ctrl) {

            element.addClass("paginationTable")

            scope.recPerPageList = [10,20,50,100];
            scope.recPerPage = scope.recPerPageList[0];
            scope.currentPage = 0;
            scope.pageList = 1;
            scope.pageHide = true;

            scope.pageSelect = function(type) {

                if(type=='previous') {
                    scope.currentPage = (scope.currentPage <= 1) ? 1 : scope.currentPage - 1;
                } else if (type=='next') {
                    scope.currentPage = (scope.currentPage >= scope.pageList) ? scope.pageList : scope.currentPage + 1;
                } else if (type=='first') {
                    scope.currentPage = 1;
                } else if (type=='last') {
                    scope.currentPage = (scope.pageList > 0) ? scope.pageList : 1;
                }
                scope.calculatePageRange();

            }

            scope.recPerPageSelect = function(recListNo) {
                scope.recPerPage = scope.recPerPageList[recListNo];
                var html = scope.calculatePagination(scope[attrs.paginationTable]);
                var replaceElement = element[0].querySelector('.paging-section');
                angular.element(replaceElement).replaceWith($compile(html)(scope));
                scope.pageSelect('first');
            }

            scope.calculatePageRange = function() {
                if(scope.PageRecTotal > 0) {
                    scope.PageRecFrom = (scope.recPerPage * (scope.currentPage - 1)) + 1;
                    scope.PageRecFrom = (scope.PageRecFrom <= 0) ? 0 : scope.PageRecFrom;
                    scope.PageRecTo = (scope.recPerPage * scope.currentPage);
                    scope.PageRecTo = (scope.PageRecTo < scope.PageRecTotal) ? scope.PageRecTo : scope.PageRecTotal;
                }
            }

            scope.calculatePageList = function() {
                scope.pageList = Math.ceil(scope.PageRecTotal/scope.recPerPage);
                scope.pageList = (scope.pageList <= 0) ? 1 : scope.pageList;
                scope.currentPage = (scope.currentPage > scope.pageList) ? 1 : scope.currentPage;
                scope.calculatePageRange();
            }

            scope.calculatePagination = function(listObj) {

                let recPerPageList = '';
                scope.PageRecFrom = 0;
                scope.PageRecTo = 0;
                scope.PageRecTotal = (listObj) ? listObj.length : 0;

                scope.calculatePageList();

                for(var i=0; i<scope.recPerPageList.length; i++) {
                    recPerPageList = recPerPageList+
                                    '<li ng-class="{'+
                                           '\'active\': recPerPage=='+scope.recPerPageList[i]+','+
                                        '}"> '+
                                    '   <a class="pointer-link" ng-click="recPerPageSelect('+i+')">'+
                                            scope.recPerPageList[i]+
                                    '   </a>'+
                                    '</li>';
                }

                return html = '<div class="paging-section" id="paging-section"> '+
                            '    <div class="page-selection"> '+
                            '        <nav aria-label="Page navigation"> '+
                            '           <div class="recperpage-container pagination">'+
                            '            <ul class="pagination custom-pagination recperpagehead"> '+
                            '               <li>'+
                            '                   <a class="pointer-link">'+
                            '                       {{PageRecFrom}} - {{PageRecTo}} of {{PageRecTotal}}'+
                            '                   </a>'+
                            '               </li>'+
                            '            </ul> '+
                            '            <ul class="pagination custom-pagination recperpagelist"> '+
                            '                '+recPerPageList+' '+
                            '            </ul> '+
                            '           </div>'+
                            '        </nav> '+
                            '    </div> '+
                            '    <div class="pagination-container"> '+
                            '        <nav aria-label="Page navigation"> '+
                            '            <ul class="pagination"> '+
                            '               <li> '+
                            '                   <a class="pointer-link" ng-click="pageSelect(\'first\')" aria-label="first"> '+
                            '                     <span class="glyphicon glyphicon-fast-backward" aria-hidden="true"></span> '+
                            '                   </a> '+
                            '               </li> '+
                            '               <li> '+
                            '                   <a class="pointer-link" ng-click="pageSelect(\'previous\')" aria-label="Previous"> '+
                            '                     <span class="glyphicon glyphicon-backward" aria-hidden="true"></span> '+
                            '                   </a> '+
                            '               </li> '+
                            '            </ul> '+
                            '            <ul class="pagination"> '+
                            '               <li><label class="pagination-text custom-label">{{currentPage}} of {{pageList}}</label></li> '+
                            '            </ul> '+
                            '            <ul class="pagination"> '+
                            '               <li> '+
                            '                   <a class="pointer-link" ng-click="pageSelect(\'next\')" aria-label="Next"> '+
                            '                     <span class="glyphicon glyphicon-forward" aria-hidden="true"></span> '+
                            '                   </a> '+
                            '               </li> '+
                            '               <li> '+
                            '                   <a class="pointer-link" ng-click="pageSelect(\'last\')" aria-label="last"> '+
                            '                     <span class="glyphicon glyphicon-fast-forward" aria-hidden="true"></span> '+
                            '                   </a> '+
                            '               </li> '+
                            '            </ul> '+
                            '        </nav> '+
                            '    </div> '+
                            '</div>';

            }

            scope.$watch(attrs.paginationTable, function (newVal, oldVal) {
                if (newVal && newVal.length > 0) {
                    scope.$watch('pageList', function(newVal, oldVal) {
                        if(newVal != oldVal || scope.currentPage == 0) {
                            scope.currentPage = 1;
                        }
                    }, true)
                    // if ((oldVal && newVal.length != oldVal.length) || !oldVal) {
                    //     scope.currentPage = (scope[attrs.paginationTable].length == 0) ? 0 : 1;
                    // }
                    var html = scope.calculatePagination(newVal);
                } else {
                    var html = "";
                }

                var replaceElement = element[0].querySelector('.paging-section');
                if (replaceElement) {
                    angular.element(replaceElement).replaceWith($compile(html)(scope));
                } else {
                    angular.element(element[0]).append($compile(html)(scope));
                }

            }, true)

        }
    }
}])

app.filter('paginationRecs', function() {
    return function(input, currentPage, recPerPage, scope) {
        var output = [];
        angular.forEach(input, function (item, index) {
            if((index >= recPerPage * (currentPage - 1)) && (index <= (recPerPage * currentPage) - 1)) {
                output.push(item);
            }
        })

        if(Object.prototype.toString.call(input) == "[object Array]") {
            scope.PageRecFrom = 0;
            scope.PageRecTo = 0;
            scope.PageRecTotal = input.length;
            scope.calculatePageList();
        }

        return output;
    }
});

app.directive('dropdownList', ['$compile', '$filter', function($compile, $filter) {
    /**
     * -----Usage------
        dropdown-list
        ng-model             <- ng-model
        ddl-list             <- the list of ng-repeat
        ddl-order            <- the field of the list which want to order
        ddl-value            <- ng-value
        ddl-show             <- the field of the list which want to show on dropdown list items
        ddl-show-pattern     <- the field-set of the list which want to show on dropdown list items (use instead of ddl-show)
        ddl-change           <- functions from ng-click or ng-change
        ddl-default          <- default ddl-show on dropdown list item (usually is the blank item)
        ddl-default-value    <- value of ddl-default

        <span dropdown-list
            ng-model=""
            ddl-list=""
            ddl-order=""
            ddl-value=""
            ddl-show=""
            ddl-show-pattern=""
            ddl-change=""
            ddl-default=""
            ddl-default-value="">
        </span>
     */
    return {
        restrict: 'AE',
        require:'ngModel',
        link: function (scope, element, attrs, ctrl) {
            scope.$watch(attrs.ddlList, function(newVal, oldVal) {
                if(newVal && newVal.length > 0) {
                    var listhtml = '';
                    var modelValue = '';
                    var clickAction = '';
                    var showText = '';
                    
                    //Declare scope values
                    // scope[attrs.ngModel] = (typeof(attrs.ddlDefaultValue)!='undefined') ? attrs.ddlDefaultValue : null;
                    (typeof(attrs.ddlDefaultValue)!='undefined') ? scope[attrs.ngModel] = attrs.ddlDefaultValue : null;
                    scope.ddlValues = (typeof(scope.ddlValues)=='undefined') ? {} : scope.ddlValues;
                    scope.ddlValues[attrs.ngModel] = [];

                    newVal = $filter('orderBy')(newVal, (attrs.ddlOrder)?attrs.ddlOrder:null);
                    clickAction = (attrs.ddlChange && attrs.ddlChange!='') ? attrs.ddlChange : '';
                    
                    //DDL default/blank
                    if(attrs.ddlDefault) {
                        listhtml = '<li ng-class="{\'list-group-item\': true, \'custom-ddl-item\': true, \'selected\': '+attrs.ngModel+'==\''+attrs.ddlDefaultValue+'\'}"'+
                                    '     ng-click="'+attrs.ngModel+'=\''+attrs.ddlDefaultValue+'\'; '+clickAction+'">'+
                                    '     <label class="custom-label">'+attrs.ddlDefault+'</label>'+
                                    '</li>';

                        scope.ddlValues[attrs.ngModel].push({
                            "val": attrs.ddlDefaultValue, 
                            "text": attrs.ddlDefault
                        });
                    }

                    if(attrs.ddlShowPattern) {
                        var textParts = attrs.ddlShowPattern.split(/[{}]{1,2}/);
                        textParts.length--;
                        textParts.shift();
                    }

                    //DDL items generation
                    newVal.forEach(function(item, index) {
                        realIndex = scope[attrs.ddlList].indexOf(item);
                        modelValue = (attrs.ddlValue && attrs.ddlValue!='') ? attrs.ddlList+'['+realIndex+'].'+attrs.ddlValue : attrs.ddlList+'['+realIndex+']';
                        showText = '';
                        
                        if(attrs.ddlShowPattern) {
                            textParts.forEach(function(part) { showText = showText + ((typeof(item[part])!='undefined') ? item[part] : part) });
                        } else if(attrs.ddlShow) {
                            showText = item[attrs.ddlShow];
                        }
                             
                        listhtml = listhtml + '<li ng-class="{\'list-group-item\': true, \'custom-ddl-item\': true, \'selected\': '+attrs.ngModel+'=='+modelValue+'}"'+
                                              '     ng-click="'+attrs.ngModel+'='+modelValue+'; '+clickAction+'">'+
                                              '     <label class="custom-label">'+showText+'</label>'+
                                              '</li>'

                        scope.ddlValues[attrs.ngModel].push({
                            "val": (attrs.ddlValue && attrs.ddlValue!='') ? item[attrs.ddlValue] : item, 
                            "text": showText
                        });
                        
                    })

                    //Final html string
                    var html = '<div class="custom-ddl-container ' + attrs.btncolor +'">'+
                        '   <button class="custom-ddl-btn">'+
                                '       <span ng-repeat="item in ddlValues.'+attrs.ngModel+' '+
                                '                       | filter:{val:(('+attrs.ngModel+'!=undefined)?'+attrs.ngModel+':null)}:true '+
                                '                       | range:1">'+
                                '           {{item.text}}'+
                                '       </span>&nbsp;'+
                                '   </button>'+
                                '   <ul class="list-group custom-ddl">'+
                                        listhtml +
                                '   </ul>'+
                                '</div>';
                } else {
                    var html = '<div class="custom-ddl-container ' + attrs.btncolor +'">'+
                                '   <button class="custom-ddl-btn">'+
                                        ((attrs.ddlDefault)?attrs.ddlDefault:'&nbsp;')+
                                '   </button>'+
                                '   <ul class="list-group custom-ddl">'+
                                '       <li ng-class="{\'list-group-item\': true, \'custom-ddl-item\': true}">'+
                                '           <label class="custom-label">'+((attrs.ddlDefault)?attrs.ddlDefault:'&nbsp;')+'</label>'+
                                '       </li>'
                                '   </ul>'+
                                '</div>';
                }
                if(element[0].querySelector('.custom-ddl-container')) {
                    var replaceElement = element[0].querySelector('.custom-ddl-container');
                    angular.element(replaceElement).replaceWith($compile(html)(scope));
                } else {
                    angular.element(element[0]).append($compile(html)(scope));
                }
                
            }, true)
        }
    }
}]);

app.directive('fyYear', ['$filter', 'BaseService', function ($filter, BaseService) {
    return {
        restrict: 'A',
        require: 'ngModel',
        link: function (scope, element, attrs, ctrl) {
            BaseService.GetFYObject(scope);

            if (attrs.fyYear == 'object') {
                let date = new Date();
                scope[attrs.ngModel] = $filter('filter')(scope.FY, { fullYear: date.getFullYear() }, true)[0];
                scope[attrs.monthModel] = $filter('filter')(scope.FYMonths, { Month: date.getMonth()+1 }, true)[0];
            }
            
        }
    }
}]);

app.directive('passwordExpireNotifyBar', ['$filter', function ($filter) {
    return {
        restrict: 'A',
        link: function (scope, element, attrs, ctrl) {
            scope.ShowExpireNotifyBar = true;
            scope.CloseNotifyBar = function () {
                sessionStorage.ExpireNotifyBar = JSON.stringify({
                    "username": attrs.username,
                    "IsClose": true
                })
                scope.ShowExpireNotifyBar = false;
            }

            if(sessionStorage.ExpireNotifyBar) {
                let ExpireNotifyBar = JSON.parse(sessionStorage.ExpireNotifyBar);
                scope.ShowExpireNotifyBar = (ExpireNotifyBar.username != attrs.username);
                // console.log(scope.ShowExpireNotifyBar);
            }
        }
    }
}])

app.directive('estimatedPayment', ['BizDataService','$compile', function (BizDataService,$compile) {
    return {
        restrict: 'AE',
        link: function(scope, element, attrs, ctrl) {
            BizDataService.GetEstimatedPayment().then(function(result) {
                scope.PaymentList = result;
            })
            scope.$watch('PaymentList', function(v) {
                if(v) {
                    let html = '<div style="height:10px"></div>'+
                                    '<label class="subtitle">Estimate payment date</label>'+
                                '<table class="AutoTable" id="tblTrans" cellpadding="2" cellspacing="2">'+
                                    '<thead>'+
                                        '<tr>'+
                                            '<th valign="top" align="center" style="width:75px;line-height:normal">'+
                                                'Biz GM approve' +
                                            '</th>'+
                                            '<th valign="top" align="center" style="width:75px;line-height:normal">'+
                                                'Account approve'+
                                            '</th>'+
                                            '<th valign="top" align="center" style="width:75px;line-height:normal">'+
                                                'Payment date'+
                                            '</th>'+
                                        '</tr>'+
                                    '</thead>'+
                                    '<tbody>'+
                                        '<tr ng-repeat="row in PaymentList | filter: {SHOW_MAIN: \'Y\'} | orderBy: PAY_DATE: true " style="line-height:normal">'+
                                            '<td style="width:75px;font-size:8.5pt;text-align:center;">' +
                                                '{{row._GM_APPROVE_DUE_DATE}}' +
                                            '</td>'+
                                            '<td style="width:75px;font-size:8.5pt;text-align:center;">'+
                                                '{{row._DUE_DATE}}'+
                                            '</td>'+
                                            '<td style="width:75px;font-size:8.5pt;text-align:center;">'+
                                                '{{row._PAY_DATE}}'+
                                            '</td>'+
                                        '</tr>'+
                                    '</tbody>'+
                                '</table>';
                    angular.element(element[0]).append($compile(html)(scope));
                }
            }, true)
        }
    }
}])
app.directive('systemMenu', ['BizDataService', '$filter', function(BizDataService, $filter) {
    return {
        restrict: 'A',
        link: function (scope, element, attrs, ctrl) {
            scope.System = {};
            if(!sessionStorage.systemMenu) {
                BizDataService.GetSystemMenu().then(function(result) {
                    scope.SystemMenu = result;
                    scope.SystemMenuTitle = $filter('unique')(result, 'HEADER_ID');
                    sessionStorage.systemMenu = JSON.stringify(scope.SystemMenu);
                    sessionStorage.systemMenuTitle = JSON.stringify(scope.SystemMenuTitle);
                });
            } else if(sessionStorage.systemMenu) {
                scope.SystemMenu = JSON.parse(sessionStorage.systemMenu);
                scope.SystemMenuTitle = JSON.parse(sessionStorage.systemMenuTitle);
            }

            scope.$watch('SystemMenu', function (newVal, oldVal) {
                if (newVal) {
                    const menuObject = $filter('filter')(scope.SystemMenu, { LINK: window.location.pathname }, true)[0];
                    if (menuObject) {
                        scope.System.Viewable = menuObject.VIEWABLE;
                        scope.System.Editable = menuObject.EDITABLE;
                    }
                }
            }, true)

            scope.systemMenuClass = function(obj) {
                return {
                    "menu-link": true,
                    "link-active": (window.location.pathname).replace(/^[\/]+|[\/]+$/g, "")==(obj.LINK).replace(/^[\/]+|[\/]+$/g, "")
                }
            }
        }
    }
}]);

app.directive('sumObjectAmount', ['$filter', function($filter) {
    /**
     * Usage
     *  sum-object-amount <- *Required
     *  obj               <- *Required: Object
     *  filter            <- *Required: Filter for object
     *  field             <- *Required: Field for sum amount (*numeric required)
     *  field-pattern     <- *sum amount by pattern group.
     *  round             <- Not required: round of decimal
     * 
     * <TAGNAME
     *  sum-object-amount
     *  obj=""
     *  field=""
     *  field-pattern
     *  round=""/>
     */
    return {
        restrict: 'A',
        link: function (scope, element, attrs, ctrl) {
            scope.$watch(attrs.obj, function(newVal, oldVal) {
                if(newVal!=null) {
                    let sumAmount = 0
                    if(attrs.field && attrs.field != '') {
                        let thisObject = newVal;
                        if(attrs.filter) {
                            thisObject = $filter('filter')(thisObject, JSON.parse(attrs.filter), true);
                        }
                        sumAmount = thisObject.reduce(function(a, b) {
                            return a+b[attrs.field]
                        },0)
                    } else if(attrs.fieldPattern && attrs.fieldPattern != '') {
                        let thisObject = newVal;
                        let thisOperation = null;
                        let sumList = [];
                        let textParts = attrs.fieldPattern.split(/[{}]{1,2}/);

                        textParts.length--;
                        textParts.shift();
                        if(attrs.filter) {
                            thisObject = $filter('filter')(thisObject, JSON.parse(attrs.filter), true);
                        }

                        let sumFromText = '';

                        textParts.forEach(function(item) {
                            item = item.trim();
                            if(item != '+' && item != '-' && item != '*' && item != '/') {
                                sumFromText = sumFromText + (thisObject.reduce(function(a, b) {
                                                                return a+b[item]
                                                            },0) || 0).toString();
                            } else {
                                sumFromText = sumFromText + item;
                            }
                        })

                        var fnSumFromText = new Function('return ' + sumFromText);
                        sumAmount = fnSumFromText();
                        if(isNaN(sumAmount) || !sumAmount) {
                            sumAmount = 0;
                        }

                    }
                    
                    if(typeof(attrs.msControlDetailColor) != 'undefined') {
                        if(sumAmount < 0) {
                            element[0].classList.add("blue");
                        }
                    }

                    element[0].innerHTML = $filter('number')(sumAmount,(attrs.round)?attrs.round:2);
                } else {
                    element[0].innerHTML = $filter('number')(0,(attrs.round)?attrs.round:2);
                }
            },true)
        },
    }
}]);

app.directive('msControlDetailAmount', ['$filter', function ($filter) {
    return {
        restrict: 'A',
        link: function (scope, element, attrs, ctrl) {
            if(attrs.msControlDetailAmount < 0) {
                element[0].classList.add("blue");
            }
            
            element[0].innerHTML = $filter('number')(attrs.msControlDetailAmount, 2);
        }
    }
}])

// app.directive('customDatePicker', ['$parse', function ($parse) {
//     return {
//         compile: function (tElm,tAttrs) {
//             return function (scope, element, attrs) {
//                 scope.dateFormat = 'd!-MMM-yy';
//                 scope.dateFormatValue = 'dd-MMM-yy';

//                 element.addClass("input-group")
//                 element.addClass("date")
//                 element.addClass("custom-datebox")
//                 element.bind("ng-click", "openStartDate()");

//                 var span = angular.element(document.createElement("span"))
//                 span.addClass("input-group-addon")
//                 var spanIcon = angular.element(document.createElement("i"))
//                 spanIcon.addClass("glyphicon")
//                 spanIcon.addClass("glyphicon-th")
//                 span.append(spanIcon);
//                 element.append(span)


//                 // element.bind('datetimepicker-neutral-timezone')
//                 // element.bind('current-date')
//                 // element.bind('type', 'text')
//                 // element.bind('class', 'form-control')
//                 // element.bind('ng-model', 'formData.START_DATE')

//             }
//         }
//     }
// }])

app.directive('datetimepickerNeutralTimezone', function () {
    return {
        restrict: 'A',
        priority: 1,
        require: 'ngModel',
        link: function (scope, element, attrs, ctrl) {

            scope.dateFormat = 'd!-MMM-yy';
            element[0].readOnly = true;

            ctrl.$formatters.push(function (value) {
                var date = new Date(Date.parse(value));
                date = new Date(date.getTime() + (60000 * date.getTimezoneOffset()));
                return date;
            });

            ctrl.$parsers.push(function (value) {
                var date = new Date(value.getTime() - (60000 * value.getTimezoneOffset()));
                return date;
            });

        }
    };
});

app.directive('datepickerOptions', ['BaseService', function (BaseService) {
    return {
        restrict: 'A',
        priority: 2,
        require: 'ngModel',
        require: '^?form',
        link: function (scope, element, attrs, ctrl) {
            // if(attrs.datepickerMaxDate) {
            //     scope.$watch(attrs.datepickerMaxDate, function(newVal, oldVal) {
            //         if(newVal) {
            //             scope[attrs.datepickerOptions] = BaseService.ObjectAssign(scope[attrs.datepickerOptions], {
            //                 "maxDate": new Date(Date.parse(newVal)),
            //             })

            //         }
            //     }, true)
            // } 
            // else 
            if(attrs.datepickerMinDate) {
                scope.$watchGroup([attrs.datepickerMinDate, attrs.ngModel], function(newVal) {
                    if(newVal[0]) {
                        scope[attrs.datepickerOptions] = BaseService.ObjectAssign(scope[attrs.datepickerOptions], {
                            "minDate": new Date(Date.parse(newVal[0])),
                        })
                    }
                    if(newVal[0] > newVal[1]) {
                        console.log(ctrl);
                        ctrl.$formatters.push(function (value) {
                            console.log(value);
                            return newVal[0];
                        });
                    }
                }, true)
            }
            
            scope[attrs.datepickerOptions] = BaseService.ObjectAssign(scope[attrs.datepickerOptions],{
                showButtonBar: false
            });

        }
    }
}]);

app.directive('disabledWeekends', ['BaseService', function (BaseService) {
    return {
        restrict: 'A',
        priority: 2,
        link: function (scope, element, attrs, ctrl) {
            //Disable weekends
            scope[attrs.datepickerOptions] = BaseService.ObjectAssign(scope[attrs.datepickerOptions],{
                dateDisabled: function (data) {
                    return (data.mode==='day' && (data.date.getDay() === 0 || data.date.getDay() === 6));
                }
            });
            
        }
    }
}]);

app.directive('currentDate', ['$parse', '$interval', function ($parse, $interval) {
    return {
        restrict: 'AS',
        require: 'ngModel',
        link: function (scope, element, attrs, ctrl) {
            
            var stop = $interval(function () {
                scope.$watch(attrs.ngModel, function (newVal, oldVal) {
                    if(!newVal) {
                        model = $parse(attrs.ngModel);
                        model.assign(scope, new Date());
                    } else {
                        $interval.cancel(stop);
                    }
                }, true)
            }, 1000)
        }
    }
}])

app.directive('currentMonth', ['$parse', '$interval', function ($parse, $interval) {
    return {
        restrict: 'AS',
        require: 'ngModel',
        link: function (scope, element, attrs, ctrl) {
            
            var stop = $interval(function () {
                scope.$watch(attrs.ngModel, function (newVal, oldVal) {
                    if(!newVal) {
                        model = $parse(attrs.ngModel);
                        let currentDate = new Date();
                        model.assign(scope, new Date(currentDate.getFullYear(), currentDate.getMonth(), 1));
                    } else {
                        $interval.cancel(stop);
                    }
                }, true)
            }, 1000)
        }
    }
}])

app.directive('formatNumeric', function () {
    return {
        restrict: 'A',
        priority: 1,
        require: 'ngModel',
        link: function (scope, element, attrs, ctrl) {
            ctrl.$parsers.push(function (value) {
                var clean = value.replace( /([^0-9.-]+)/g, '');
                if (value !== clean) {
                    ctrl.$setViewValue(clean);
                    ctrl.$render();
                }
                return parseFloat(clean || 0);
            });
            element.bind('keypress', function(event) {
                if(event.keyCode === 32) {
                    event.preventDefault();
                }
            });

            element.bind('click', function(event) {
                this.select();
            })

            //element[0].style.textAlign = "right";
            //element[0].style.width = "100px";
        }
    }
});

app.directive('formatDecimal', ['$filter', function ($filter) {
    return {
        restrict: 'A',
        priority: 1,
        require: 'ngModel',
        link: function (scope, element, attrs, ctrl) {

            ctrl.$parsers.push(function (value) {
                var clean = value.replace(/([^0-9.]|\.(?=.*\.))/g, '');
                var match = clean.match(/\d\.*\d{1,3}$/);
                if (value !== clean) {
                    ctrl.$setViewValue(clean);
                    ctrl.$render();
                }
                return clean;
            });
            element.bind('keypress', function (event) {
                if (event.keyCode === 32) {
                    event.preventDefault();
                }
            });
            
            element.bind('click', function(event) {
                this.select();
            })
            //element[0].style.textAlign = "right";
            //element[0].style.width = "100px";
        }
    }
}]);

app.directive('convertToNumber', function () {
    return {
        require: 'ngModel',
        link: function (scope, element, attrs, ngModel) {
            ngModel.$parsers.push(function (val) {
                return parseInt(val, 10);
            });
            ngModel.$formatters.push(function (val) {
                return '' + val;
            });
        }
    };
});


app.filter('max', function () {
    return function (collection, keyname) {
        var output = {},
            maxvalue = null

        angular.forEach(collection, function (item) {
            if (item[keyname] > maxvalue) {
                maxvalue = item[keyname];
                output = item;
            }
        });

        return maxvalue;
    };
});

app.filter("range", function () {
    return function (input, total) {
        var rangeFilter = [];
        total = parseInt(total);
        let count = 0;

        angular.forEach(input, function (item) {
            count++;
            if (count <= total) {
                rangeFilter.push(item);
            }
        })

        return rangeFilter;
    };
});

app.filter('unique', function () {
    return function (collection, keyname) {
        var output = [],
            keys = [];

        angular.forEach(collection, function (item) {
            var key = item[keyname];
            if (keys.indexOf(key) === -1) {
                keys.push(key);
                output.push(item);
            }
        });

        return output;
    };
});

app.filter('uniqueArray', function () {
    return function (collection) {
        var output = [],
            keys = [];

        angular.forEach(collection, function (item) {
            var key = item;
            if (keys.indexOf(key) === -1) {
                keys.push(key);
                output.push(item);
            }
        });

        return output;
    };
});

app.filter('unsafe', ['$sce', function ($sce) {
    return $sce.trustAsHtml;
}]);
app.filter('inArray', ['$filter', function($filter) {
    return function (collection, arrayFilter, keyname) {
        if(arrayFilter) {
            return $filter('filter')(collection, function(item) {
                return arrayFilter.indexOf(item[keyname]) != -1;
            })
        }
    }
}])
if (!window.location.origin) {
    window.location.origin = window.location.protocol + "//" + window.location.hostname + (window.location.port ? ':' + window.location.port: '');
}