﻿app.controller("InsuranceUploadController", ['$scope', 'BaseService', 'INSURANCE_PATH', 'Upload', '$timeout', '$filter', function ($scope, BaseService, INSURANCE_PATH, Upload, $timeout, $filter) {
    //Insurance Data here

    let date = new Date()
    $scope.periodMonth = (date.getMonth() + 1);
    $scope.periodYear = date.getFullYear();

    $scope.ListHeader = null;
    $scope.ListDetail = null;
    $scope.confirmBtn = null;
    $scope.fileUpload = null;

    $scope.uploadInsuranceData = function () {

        var file = this.fileUpload;

        $scope.fileName = file.name;
        $scope.ListDetail = null;

        file.upload = Upload.upload({
            url: window.location.origin + INSURANCE_PATH + '/UploadInsuranceData',
            data: { File: file, periodMonth: $scope.periodMonth, periodYear: $scope.periodYear },
        });

        file.upload.then(function (res) {
            $timeout(function () {
                if(res.data.result == 500) {
                    BaseService.Message.alert("Please insert file.");
                } else if (res.data.result == 200) {
                    BaseService.Message.alert((res.data.data));
                } else {
                    $scope.ListDetail = res.data;
                    $scope.confirmBtn = true;
                }
            })
        }, function (res) {
            if (res.status > 0)
            {
              $scope.errorMsg = res.status + ': ' + res.data;
              BaseService.Message.alert("Cannot upload file")
            }
        }, function (evt) {
            file.progress = Math.min(100, parseInt(100.0 * evt.loaded / evt.total));
        });
    };

    $scope.confirmUpload = function (fileName) {

        var sendData = [{ "fileName": fileName, periodMonth: $scope.periodMonth, periodYear: $scope.periodYear }];
        var data = BaseService.ConvertParam(sendData[0]);

        BaseService.CallAction(INSURANCE_PATH, "ConfirmInsuranceUpload", data)
            .then(function (result) {
                if (result.result == 200) {
                    $scope.ListDetail = null;
                    $scope.ListHeader = null;
                    $scope.confirmBtn = null;
                    $scope.fileUpload = null;
                    $scope.GetInsuranceData();
                    BaseService.Message.info("Successfully saved");
                }
            }, function (error) {
                BaseService.Message.alert("Cannot save your change, please contact administrator.")
                console.log('Unable to confirm uploaded data: ' + error.message)
            })
    }

    $scope.cancelUpload = function (fileName) {

        var sendData = [{ "fileName": fileName, periodMonth: $scope.periodMonth, periodYear: $scope.periodYear }];
        var data = BaseService.ConvertParam(sendData[0]);

        BaseService.CallAction(INSURANCE_PATH, "CancelInsuranceUpload", data)
            .then(function (result) {

                if (result.result == 200) {
                    $scope.ListDetail = null;
                    $scope.ListHeader = null;
                    $scope.confirmBtn = null;
                    $scope.fileUpload = null;
                    $scope.GetInsuranceData();
                }
            }, function (error) {
                BaseService.Message.alert("Cannot save your change, please contact administrator.")
                console.log('Unable to cancel uploaded data: ' + error.message)
            })
    }

    $scope.GetFileList = function () {

        var sendData = [{ periodMonth: $scope.periodMonth, periodYear: $scope.periodYear }];
        var data = BaseService.ConvertParam(sendData[0]);

        $scope.ListHeader = null;
        $scope.ListDetail = null;

        BaseService.GetHeaderGridView("FileHistory")
            .then(function (result) {
                $scope.ListHeader = result
            }, function (error) {
                console.log('Unable to load File header data: ' + error.message)
            })

        BaseService.CallAction(INSURANCE_PATH, "GetFileList", data)
            .then(function (result) {
                $scope.ListDetail = result;
            }, function (error) {
                console.log('Unable to list files: ' + error.message)
            })
        $scope.isFileList = true;
    }

    $scope.GetInsuranceData = function () {

        $scope.ListHeader = null;
        $scope.ListDetail = null;

        var sendData = [{ periodMonth: String('00' + $scope.periodMonth).slice(-2), periodYear: $scope.periodYear }];
        var data = BaseService.ConvertParam(sendData[0]);

        BaseService.GetHeaderGridView("InsuranceData")
            .then(function (result) {
                $scope.ListHeader = result
            }, function (error) {
                console.log('Unable to load Insurance Data header data: ' + error.message)
            })

        BaseService.CallAction(INSURANCE_PATH, "GetInsuranceData", data)
            .then(function (result) {
                $scope.ListDetail = result;
            }, function (error) {
                console.log('Unable to get Insurance data: ' + error.message)
            })

        $scope.isFileList = false;
    }
    $scope.GetInsuranceData();

    $scope.setInitRec = function (obj) {

        $scope.$watch(obj, function (newVal, oldVal, scope) {
            $scope.recordFilter = (($scope[obj] != null) ? (($scope[obj].length >= 100) ? 100 : $scope[obj].length) : 0);
        }, true)

    }
    
    $scope.GetTemplateFile = function () {
        location.href="/Doc/template/LWT template.xlsx";
    }
}])

app.controller("InsuranceAmountController", ['$scope', 'BaseService', 'INSURANCE_PATH', 'BizDataService', function ($scope, BaseService, INSURANCE_PATH, BizDataService) {

    Opening();

    function Opening() {
        let date = new Date()
        $scope.periodMonth = (date.getMonth() + 1);
        $scope.periodYear = date.getFullYear();
        $scope.formData = {};
        $scope.ListHeader = null
        $scope.ListDetail = null
        GetRetailSalesMonth();
        GetHeader();
    }

    $scope.ViewData_Click = function () {

        if (typeof ($scope.periodYear) != 'undefined' && typeof ($scope.periodMonth) != 'undefined') {
            $scope.DATA_MONTH = String($scope.periodYear) + String('00'+$scope.periodMonth).slice(-2);
        }

        if (typeof ($scope.DATA_MONTH) == 'undefined') {
            return;
        }
        GetData();
    }

    function GetRetailSalesMonth() {
        BizDataService.GetRetailSalesMonth()
            .then(function (result) {
                $scope.RetailSalesMonth = result
            }, function (error) {
                console.log('Unable to load RetailSalesMonth data: ' + error.message)
            })
    }

    function GetHeader() {
        BaseService.GetHeaderGridView('InsuranceSumRep')
            .then(function (result) {
                $scope.ListHeader = result

            }, function (error) {
                console.log('Unable to load Dealer summary data: ' + error.message)
            })
    }

    function GetData() {
        var sendData = [{ "DATA_MONTH": $scope.DATA_MONTH }];
        var data = BaseService.ConvertParam(sendData[0]);

        BaseService.CallAction(INSURANCE_PATH, "GetInsuranceSumList", data)
            .then(function (result) {
                $scope.ListDetail = result
            }, function (error) {
                console.log('Unable to load GetInsuranceSumList data: ' + error.message)
            })
    }

    $scope.OrderColumn = function (pColName) {
        $scope.sortType = pColName;
        $scope.sortReverse = !$scope.sortReverse;
    };

    $scope.setInitRec = function (obj) {
        $scope.$watch(obj, function (newVal, oldVal, scope) {
            $scope.recordFilter = (($scope[obj] != null) ? (($scope[obj].length >= 100) ? 100 : $scope[obj].length) : 0);
        }, true)

    }
}])

app.controller("InsuranceReportController", ['$scope', 'BaseService', 'INSURANCE_PATH', 'REPORT_PATH', 'BizDataService', '$filter', function ($scope, BaseService, INSURANCE_PATH, REPORT_PATH, BizDataService, $filter) {

    Opening();

    function Opening() {
        let date = new Date()
        $scope.periodMonth = (date.getMonth() + 1);
        $scope.periodYear = date.getFullYear();
        $scope.ListHeader = {};
        $scope.ListDetail = [];
        $scope.sortType = '';

        GetHeader();
        $scope.$watch('ViewData_Click', function(value) {
            if(value) {
                $scope.ViewData_Click();
            }
        },true)
    }

    $scope.ViewData_Click = function () {

        if (typeof ($scope.periodYear) != 'undefined' && typeof ($scope.periodMonth) != 'undefined') {
            $scope.DATA_MONTH = String($scope.periodYear) + String('00' + $scope.periodMonth).slice(-2);
        }

        if (typeof ($scope.DATA_MONTH) == 'undefined') {
            return;
        }

        GetData();
    }

    function GetData() {
        var data = BaseService.ConvertParam({
            "DATA_MONTH": $scope.DATA_MONTH,
        });

        BaseService.CallAction(INSURANCE_PATH, "InsuranceMatchChassis", data)
            .then(function (result) {
                $scope.ListDetail = result;
            }, function (error) {
                console.log(error);
            })
    }

    function GetHeader() {
        BaseService.GetHeaderGridView('InsuranceReport')
            .then(function (result) {
                $scope.ListHeader = result;
            }, function (error) {
                console.log('Unable to load header data: ' + error.message)
            })
    }

    $scope.OrderColumn = function (pColName) {
        $scope.sortType = pColName;
        $scope.sortReverse = !$scope.sortReverse;
    };

    $scope.Report_Click = function () {
        $scope.reportCode='VME_Rpt_Insurance_Matched_Detail';
        $scope.reportName='NPP Report';
        window.open(window.location.origin + REPORT_PATH + 'rt=1&rc=' + $scope.reportCode + '&p1=' + $scope.DATA_MONTH + '&p2=' + $scope.reportName, '_blank');
    }

}])
